#include "statefile_wrapper.h"
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <extapp_api.h>
#include "lz4.h"

#define MAX_FILE_SIZE 8192
#define MAX_SCRIPTSTORE_SIZE 4096

extern bool file_exists(const char * filename);
extern bool erase_file(const char * filename);
extern const char * read_file(const char * filename);
extern bool write_file(const char * filename, const char * s, size_t len);

typedef struct {
  char *fd;
  size_t pos,maxpos;
  int isWrite;
  char name[256];
} statefile_desc_t;

#define LZ4 // compress statefile

FILE *statefile_fopen(const char *pathname, const char *mode) {
  return 0; // disabled, restoring does not work
  extapp_pushRectUniform(0, 100, LCD_WIDTH, LCD_HEIGHT, 0);
  extapp_drawTextSmall(pathname, 0, 100, 65535, 14567, false);
  printf("Wrapper: open %s mode %s\n", pathname, mode);
  statefile_desc_t *s=calloc(sizeof(statefile_desc_t), 1);
  if (!s) goto err;
  strncpy(s->name, pathname, 256);
  if (mode[0]=='w') {
    extapp_fileErase(pathname, EXTAPP_RAM_FILE_SYSTEM);
    s->fd=calloc(MAX_FILE_SIZE, 1);
    s->isWrite=1;
    return (FILE*)s;
  }
  if (mode[0]=='r' && extapp_fileExists(pathname, EXTAPP_RAM_FILE_SYSTEM)){
    size_t len = 0;
#ifdef LZ4
    extapp_pushRectUniform(0, 80, LCD_WIDTH, 20, 0);
    extapp_drawTextSmall("File exists", 0, 80, 65535, 14567, false);
    s->fd = malloc((size_t)MAX_FILE_SIZE);
    const char *compressed_data = extapp_fileRead(pathname, &len, EXTAPP_RAM_FILE_SYSTEM);
    extapp_pushRectUniform(0, 40, LCD_WIDTH, 20, 0);
    extapp_drawTextSmall("Decompressing", 0, 40, 65535, 14567, false);
    const int decompressed_size = LZ4_decompress_safe(compressed_data, s->fd,
						      len, MAX_FILE_SIZE);
    if (decompressed_size>0){
      extapp_pushRectUniform(0, 60, LCD_WIDTH, 20, 0);
      extapp_drawTextSmall("Decompressed OK", 0, 60, 65535, 14567, false);
      extapp_getKey(0,0);
      return (FILE*)s;
    } 
    free(s->fd);
    extapp_pushRectUniform(0, 60, LCD_WIDTH, 20, 0);
    extapp_drawTextSmall("Decompressed size<=0", 0, 60, 65535, 14567, false);
#else
    s->fd = malloc((size_t)MAX_FILE_SIZE); 
    const char * data = extapp_fileRead(pathname, &len, EXTAPP_RAM_FILE_SYSTEM);
    if (data){
      char msg[32]="Got 00000 bytes";
      msg[4]='0'+len/10000;
      msg[5]='0'+((len/1000) % 10);
      msg[6]='0'+((len/100) % 10);
      msg[7]='0'+((len/10) % 10);
      msg[8]='0'+(len % 10);
      extapp_pushRectUniform(0, 120, LCD_WIDTH, 20, 0);
      extapp_drawTextSmall(msg, 0, 120, 65535, 14567, false);
      extapp_getKey(0,0);
      memcpy(s->fd,data,len);
      return (FILE*)s;
    }
    else 
      free(s->fd);
#endif
  }
err:
  extapp_pushRectUniform(0, 120, LCD_WIDTH, 20, 0);
  extapp_drawTextSmall("Wrapper: open failed", 0, 120, 65535, 14567, false);
  extapp_getKey(0,0);
  printf("Wrapper: open failed\n");
  free(s);
  return NULL;
}

int statefile_fclose(FILE *stream) {
  statefile_desc_t *s=(statefile_desc_t*)stream;
  if (s->isWrite) {
#ifdef LZ4
    char* compressed_data = malloc((size_t) MAX_SCRIPTSTORE_SIZE);
    const int compressed_data_size = LZ4_compress_default(s->fd, compressed_data,
							  s->maxpos, MAX_SCRIPTSTORE_SIZE);
    if (compressed_data_size > 0) {
      printf("We successfully compressed data: %d\n", compressed_data_size);
      extapp_pushRectUniform(0, 120, LCD_WIDTH, 20, 0);
      extapp_drawTextSmall("Compression success", 0, 120, 65535, 14567, false);
      if (!extapp_fileWrite(s->name, compressed_data, compressed_data_size, EXTAPP_RAM_FILE_SYSTEM)){
	extapp_pushRectUniform(0, 140, LCD_WIDTH, 20, 0);
	extapp_drawTextSmall("Write error", 0, 140, 65535, 14567, false);
      }
      extapp_getKey(0,0);
    }
    else {
      extapp_pushRectUniform(0, 120, LCD_WIDTH, 20, 0);
      extapp_drawTextSmall("Wrapper: close failed", 0, 120, 65535, 14567, false);
      extapp_getKey(0,0);
    }		  
    free(compressed_data);
    
#else // LZ4
    extapp_pushRectUniform(0, 120, LCD_WIDTH, 20, 0);
    char msg[32]="Wrote 00000 bytes";
    msg[6]='0'+s->maxpos/10000;
    msg[7]='0'+((s->maxpos/1000) % 10);
    msg[8]='0'+((s->maxpos/100) % 10);
    msg[9]='0'+((s->maxpos/10) % 10);
    msg[10]='0'+(s->maxpos % 10);
    extapp_pushRectUniform(0, 120, LCD_WIDTH, 20, 0);
    extapp_drawTextSmall(msg, 0, 120, 65535, 14567, false);
    extapp_getKey(0,0);
    extapp_fileWrite(s->name, s->fd, s->maxpos, EXTAPP_RAM_FILE_SYSTEM);
#endif // LZ4
  }
  free(s->fd);
  free(s);
  return 0;
}

size_t statefile_fread(void *ptr, size_t size, size_t nmemb, FILE *stream) {
	statefile_desc_t *s=(statefile_desc_t*)stream;
	printf("Wrapper: pos %zd reading %zd\n", s->pos, size*nmemb);
	if (size*nmemb==0) return nmemb;
	memcpy(ptr, s->fd + s->pos, size*nmemb);
	s->pos+=(size*nmemb);
	return nmemb;
}

size_t statefile_fwrite(const void *ptr, size_t size, size_t nmemb, FILE *stream) {
	statefile_desc_t *s=(statefile_desc_t*)stream;
	printf("Wrapper: pos %zd writing %zd\n", s->pos, size*nmemb);
	if (size*nmemb==0) return nmemb;
	if (s->pos + size*nmemb > MAX_FILE_SIZE) return 0;
	memcpy(s->fd + s->pos, ptr, size*nmemb);
	s->pos+=(size*nmemb);
	if (s->maxpos<s->pos) s->maxpos=s->pos;
	return nmemb;
}

int statefile_fseek(FILE *stream, long offset, int whence) {
	statefile_desc_t *s=(statefile_desc_t*)stream;
	int r=s->pos;
	if (whence==SEEK_SET) {
		s->pos=offset;
	} else if (whence==SEEK_CUR) {
		s->pos+=offset;
	} else if (whence==SEEK_END) {
		abort(); //not implemented
	}
	printf("Wrapper: seek from %d to %zd\n", r, s->pos);
	return 0;
}
